@@ -1,9 +1,9 @@
 /*
- * "$Id: file.c 1701 2011-12-30 05:03:21Z mike $"
+ * "$Id: file.c 1724 2014-03-16 02:49:18Z msweet $"
  *
  *   Filename routines for HTMLDOC, a HTML document processing program.
  *
- *   Copyright 2011 by Michael R Sweet.
+ *   Copyright 2011, 2014 by Michael R Sweet.
  *   Copyright 1997-2010 by Easy Software Products.  All rights reserved.
  *
  *   This program is free software.  Distribution and use rights are outlined in
@@ -926,7 +926,7 @@ file_method(const char *s)	/* I - Filename or URL */
  */
 
 void
-file_nolocal()
+file_nolocal(void)
 {
   no_local = 1;
 }
@@ -1106,5 +1106,5 @@ file_temp(char *name,			/* O - Filename */
 
 
 /*
- * End of "$Id: file.c 1701 2011-12-30 05:03:21Z mike $".
+ * End of "$Id: file.c 1724 2014-03-16 02:49:18Z msweet $".
  */
