@@ -1,9 +1,9 @@
 /*
- * "$Id: htmldoc.cxx 1668 2011-12-23 06:23:11Z mike $"
+ * "$Id: htmldoc.cxx 1729 2016-01-02 22:58:41Z msweet $"
  *
  *   Main entry for HTMLDOC, a HTML document processing program.
  *
- *   Copyright 2011 by Michael R Sweet.
+ *   Copyright 2011, 2014 by Michael R Sweet.
  *   Copyright 1997-2010 by Easy Software Products.  All rights reserved.
  *
  *   This program is free software.  Distribution and use rights are outlined in
@@ -71,6 +71,7 @@ static double	get_seconds(void);
 static int	load_book(const char *filename, tree_t **document,
 		          exportfunc_t *exportfunc, int set_nolocal = 0);
 static void	parse_options(const char *line, exportfunc_t *exportfunc);
+static const char *prefs_getrc(void);
 static int	read_file(const char *filename, tree_t **document,
 		          const char *path);
 static void	set_permissions(const char *p);
@@ -1312,7 +1313,7 @@ main(int  argc,				/* I - Number of command-line arguments */
  * 'prefs_getrc()' - Get the rc file for preferences...
  */
 
-const char *
+static const char *
 prefs_getrc(void)
 {
 #ifdef WIN32
@@ -2520,6 +2521,8 @@ set_permissions(const char *p)		// I - Permission string
 
   if (Permissions != -4)
     Encryption = 1;
+
+  free(copyp);
 }
 
 
@@ -2701,5 +2704,5 @@ usage(const char *arg)			// I - Bad argument string
 
 
 /*
- * End of "$Id: htmldoc.cxx 1668 2011-12-23 06:23:11Z mike $".
+ * End of "$Id: htmldoc.cxx 1729 2016-01-02 22:58:41Z msweet $".
  */
