@@ -1,22 +1,19 @@
 /*
- * "$Id: http-addrlist.c 1702 2011-12-30 07:41:10Z mike $"
+ * "$Id: http-addrlist.c 1726 2016-01-02 21:46:20Z msweet $"
  *
- *   HTTP address list routines for CUPS.
+ * HTTP address list routines for HTMLDOC.
  *
- *   Copyright 2007-2011 by Apple Inc.
- *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
+ * Copyright 2016 by Michael R Sweet.
+ * Copyright 2007-2014 by Apple Inc.
+ * Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
- *   These coded instructions, statements, and computer programs are the
- *   property of Apple Inc. and are protected by Federal copyright
- *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
- *   which should have been included with this file.  If this file is
- *   file is missing or damaged, see the license at "http://www.cups.org/".
+ * These coded instructions, statements, and computer programs are the
+ * property of Apple Inc. and are protected by Federal copyright
+ * law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ * which should have been included with this file.  If this file is
+ * file is missing or damaged, see the license at "http://www.cups.org/".
  *
- * Contents:
- *
- *   httpAddrConnect()  - Connect to any of the addresses in the list.
- *   httpAddrFreeList() - Free an address list.
- *   httpAddrGetList()  - Get a list of addresses for a hostname.
+ * This file is subject to the Apple OS-Developed Software exception.
  */
 
 /*
@@ -27,56 +24,100 @@
 #ifdef HAVE_RESOLV_H
 #  include <resolv.h>
 #endif /* HAVE_RESOLV_H */
+#include <errno.h>
+#ifdef HAVE_POLL
+#  include <poll.h>
+#endif /* HAVE_POLL */
+#ifndef WIN32
+#  include <fcntl.h>
+#endif /* WIN32 */
 
 
 /*
  * 'httpAddrConnect()' - Connect to any of the addresses in the list.
  *
- * @since CUPS 1.2/Mac OS X 10.5@
+ * @since CUPS 1.2/OS X 10.5@
  */
 
 http_addrlist_t *			/* O - Connected address or NULL on failure */
 httpAddrConnect(
     http_addrlist_t *addrlist,		/* I - List of potential addresses */
     int             *sock)		/* O - Socket */
+{
+  DEBUG_printf(("httpAddrConnect(addrlist=%p, sock=%p)", addrlist, sock));
+
+  return (httpAddrConnect2(addrlist, sock, 30000, NULL));
+}
+
+
+/*
+ * 'httpAddrConnect2()' - Connect to any of the addresses in the list with a
+ *                        timeout and optional cancel.
+ *
+ * @since CUPS 1.7/OS X 10.9@
+ */
+
+http_addrlist_t *			/* O - Connected address or NULL on failure */
+httpAddrConnect2(
+    http_addrlist_t *addrlist,		/* I - List of potential addresses */
+    int             *sock,		/* O - Socket */
+    int             msec,		/* I - Timeout in milliseconds */
+    int             *cancel)		/* I - Pointer to "cancel" variable */
 {
   int			val;		/* Socket option value */
-#ifdef __APPLE__
-  struct timeval	timeout;	/* Socket timeout value */
-#endif /* __APPLE__ */
+#ifdef O_NONBLOCK
+  socklen_t		len;		/* Length of value */
+  http_addr_t		peer;		/* Peer address */
+  int			flags,		/* Socket flags */
+			remaining;	/* Remaining timeout */
+#  ifdef HAVE_POLL
+  struct pollfd		pfd;		/* Polled file descriptor */
+#  else
+  fd_set		input_set,	/* select() input set */
+			output_set;	/* select() output set */
+  struct timeval	timeout;	/* Timeout */
+#  endif /* HAVE_POLL */
+  int			nfds;		/* Result from select()/poll() */
+#endif /* O_NONBLOCK */
 #ifdef DEBUG
   char			temp[256];	/* Temporary address string */
 #endif /* DEBUG */
 
 
-  DEBUG_printf(("httpAddrConnect(addrlist=%p, sock=%p)", addrlist, sock));
+  DEBUG_printf(("httpAddrConnect2(addrlist=%p, sock=%p, msec=%d, cancel=%p)",
+                addrlist, sock, msec, cancel));
 
   if (!sock)
   {
-#ifdef WIN32
-    errno = WSAEINVAL;
-#else
     errno = EINVAL;
-#endif /* WIN32 */
-
+    _cupsSetError(IPP_STATUS_ERROR_INTERNAL, strerror(errno), 0);
     return (NULL);
   }
 
+  if (cancel && *cancel)
+    return (NULL);
+
+  if (msec <= 0 || getenv("CUPS_DISABLE_ASYNC_CONNECT"))
+    msec = INT_MAX;
+
  /*
   * Loop through each address until we connect or run out of addresses...
   */
 
   while (addrlist)
   {
+    if (cancel && *cancel)
+      return (NULL);
+
    /*
     * Create the socket...
     */
 
-    DEBUG_printf(("2httpAddrConnect: Trying %s:%d...",
+    DEBUG_printf(("2httpAddrConnect2: Trying %s:%d...",
 		  httpAddrString(&(addrlist->addr), temp, sizeof(temp)),
-		  _httpAddrPort(&(addrlist->addr))));
+		  httpAddrPort(&(addrlist->addr))));
 
-    if ((*sock = (int)socket(_httpAddrFamily(&(addrlist->addr)), SOCK_STREAM,
+    if ((*sock = (int)socket(httpAddrFamily(&(addrlist->addr)), SOCK_STREAM,
                              0)) < 0)
     {
      /*
@@ -93,46 +134,25 @@ httpAddrConnect(
     */
 
     val = 1;
-#ifdef WIN32
-    setsockopt(*sock, SOL_SOCKET, SO_REUSEADDR, (const char *)&val,
-               sizeof(val));
-#else
-    setsockopt(*sock, SOL_SOCKET, SO_REUSEADDR, &val, sizeof(val));
-#endif /* WIN32 */
+    setsockopt(*sock, SOL_SOCKET, SO_REUSEADDR, CUPS_SOCAST &val, sizeof(val));
 
 #ifdef SO_REUSEPORT
     val = 1;
-    setsockopt(*sock, SOL_SOCKET, SO_REUSEPORT, &val, sizeof(val));
+    setsockopt(*sock, SOL_SOCKET, SO_REUSEPORT, CUPS_SOCAST &val, sizeof(val));
 #endif /* SO_REUSEPORT */
 
 #ifdef SO_NOSIGPIPE
     val = 1;
-    setsockopt(*sock, SOL_SOCKET, SO_NOSIGPIPE, &val, sizeof(val));
+    setsockopt(*sock, SOL_SOCKET, SO_NOSIGPIPE, CUPS_SOCAST &val, sizeof(val));
 #endif /* SO_NOSIGPIPE */
 
-#ifdef __APPLE__
-   /*
-    * Use a 30-second read timeout when connecting to limit the amount of time
-    * we block...
-    */
-
-    timeout.tv_sec  = 30;
-    timeout.tv_usec = 0;
-    setsockopt(*sock, SOL_SOCKET, SO_RCVTIMEO, &timeout, sizeof(timeout));
-#endif /* __APPLE__ */
-
    /*
     * Using TCP_NODELAY improves responsiveness, especially on systems
     * with a slow loopback interface...
     */
 
     val = 1;
-#ifdef WIN32
-    setsockopt(*sock, IPPROTO_TCP, TCP_NODELAY, (const char *)&val,
-               sizeof(val));
-#else
-    setsockopt(*sock, IPPROTO_TCP, TCP_NODELAY, &val, sizeof(val));
-#endif /* WIN32 */
+    setsockopt(*sock, IPPROTO_TCP, TCP_NODELAY, CUPS_SOCAST &val, sizeof(val));
 
 #ifdef FD_CLOEXEC
    /*
@@ -142,48 +162,202 @@ httpAddrConnect(
     fcntl(*sock, F_SETFD, FD_CLOEXEC);
 #endif /* FD_CLOEXEC */
 
+#ifdef O_NONBLOCK
+   /*
+    * Do an asynchronous connect by setting the socket non-blocking...
+    */
+
+    DEBUG_printf(("httpAddrConnect2: Setting non-blocking connect()"));
+
+    flags = fcntl(*sock, F_GETFL, 0);
+    if (msec != INT_MAX)
+    {
+      DEBUG_puts("httpAddrConnect2: Setting non-blocking connect()");
+
+      fcntl(*sock, F_SETFL, flags | O_NONBLOCK);
+    }
+#endif /* O_NONBLOCK */
+
    /*
     * Then connect...
     */
 
-    if (!connect(*sock, &(addrlist->addr.addr),
-                 httpAddrLength(&(addrlist->addr))))
+    if (!connect(*sock, &(addrlist->addr.addr), (socklen_t)httpAddrLength(&(addrlist->addr))))
     {
-      DEBUG_printf(("1httpAddrConnect: Connected to %s:%d...",
+      DEBUG_printf(("1httpAddrConnect2: Connected to %s:%d...",
 		    httpAddrString(&(addrlist->addr), temp, sizeof(temp)),
-		    _httpAddrPort(&(addrlist->addr))));
-      break;
+		    httpAddrPort(&(addrlist->addr))));
+
+#ifdef O_NONBLOCK
+      fcntl(*sock, F_SETFL, flags);
+#endif /* O_NONBLOCK */
+
+      return (addrlist);
     }
 
-    DEBUG_printf(("1httpAddrConnect: Unable to connect to %s:%d: %s",
+#ifdef O_NONBLOCK
+#  ifdef WIN32
+    if (WSAGetLastError() == WSAEINPROGRESS ||
+        WSAGetLastError() == WSAEWOULDBLOCK)
+#  else
+    if (errno == EINPROGRESS || errno == EWOULDBLOCK)
+#  endif /* WIN32 */
+    {
+      DEBUG_puts("1httpAddrConnect2: Finishing async connect()");
+
+      fcntl(*sock, F_SETFL, flags);
+
+      for (remaining = msec; remaining > 0; remaining -= 250)
+      {
+	do
+        {
+          if (cancel && *cancel)
+          {
+	   /*
+	    * Close this socket and return...
+	    */
+
+            DEBUG_puts("1httpAddrConnect2: Canceled connect()");
+
+            httpAddrClose(NULL, *sock);
+
+	    *sock = -1;
+
+	    return (NULL);
+          }
+
+#  ifdef HAVE_POLL
+	  pfd.fd     = *sock;
+	  pfd.events = POLLIN | POLLOUT;
+
+          nfds = poll(&pfd, 1, remaining > 250 ? 250 : remaining);
+
+	  DEBUG_printf(("1httpAddrConnect2: poll() returned %d (%d)", nfds,
+	                errno));
+
+#  else
+	  FD_ZERO(&input_set);
+	  FD_SET(*sock, &input_set);
+	  output_set = input_set;
+
+	  timeout.tv_sec  = 0;
+	  timeout.tv_usec = (remaining > 250 ? 250 : remaining) * 1000;
+
+	  nfds = select(*sock + 1, &input_set, &output_set, NULL, &timeout);
+
+	  DEBUG_printf(("1httpAddrConnect2: select() returned %d (%d)", nfds,
+	                errno));
+#  endif /* HAVE_POLL */
+	}
+#  ifdef WIN32
+	while (nfds < 0 && (WSAGetLastError() == WSAEINTR ||
+			    WSAGetLastError() == WSAEWOULDBLOCK));
+#  else
+	while (nfds < 0 && (errno == EINTR || errno == EAGAIN));
+#  endif /* WIN32 */
+
+        if (nfds > 0)
+        {
+          len = sizeof(peer);
+          if (!getpeername(*sock, (struct sockaddr *)&peer, &len))
+          {
+	    DEBUG_printf(("1httpAddrConnect2: Connected to %s:%d...",
+			  httpAddrString(&peer, temp, sizeof(temp)),
+			  httpAddrPort(&peer)));
+
+	    return (addrlist);
+	  }
+
+          break;
+        }
+      }
+    }
+#endif /* O_NONBLOCK */
+
+    DEBUG_printf(("1httpAddrConnect2: Unable to connect to %s:%d: %s",
 		  httpAddrString(&(addrlist->addr), temp, sizeof(temp)),
-		  _httpAddrPort(&(addrlist->addr)), strerror(errno)));
+		  httpAddrPort(&(addrlist->addr)), strerror(errno)));
+
+#ifndef WIN32
+    if (errno == EINPROGRESS)
+      errno = ETIMEDOUT;
+#endif /* !WIN32 */
 
    /*
     * Close this socket and move to the next address...
     */
 
-#ifdef WIN32
-    closesocket(*sock);
-#else
-    close(*sock);
-#endif /* WIN32 */
+    httpAddrClose(NULL, *sock);
 
     *sock    = -1;
     addrlist = addrlist->next;
   }
 
   if (!addrlist)
-    _cupsSetError(IPP_SERVICE_UNAVAILABLE, strerror(errno), 0);
+#ifdef WIN32
+    _cupsSetError(IPP_STATUS_ERROR_SERVICE_UNAVAILABLE, "Connection failed", 0);
+#else
+    _cupsSetError(IPP_STATUS_ERROR_SERVICE_UNAVAILABLE, strerror(errno), 0);
+#endif /* WIN32 */
 
   return (addrlist);
 }
 
 
+
+/*
+ * 'httpAddrCopyList()' - Copy an address list.
+ *
+ * @since CUPS 1.7/OS X 10.9@
+ */
+
+http_addrlist_t	*			/* O - New address list or @code NULL@ on error */
+httpAddrCopyList(
+    http_addrlist_t *src)		/* I - Source address list */
+{
+  http_addrlist_t	*dst = NULL,	/* First list entry */
+			*prev = NULL,	/* Previous list entry */
+			*current = NULL;/* Current list entry */
+
+
+  while (src)
+  {
+    if ((current = malloc(sizeof(http_addrlist_t))) == NULL)
+    {
+      current = dst;
+
+      while (current)
+      {
+        prev    = current;
+        current = current->next;
+
+        free(prev);
+      }
+
+      return (NULL);
+    }
+
+    memcpy(current, src, sizeof(http_addrlist_t));
+
+    current->next = NULL;
+
+    if (prev)
+      prev->next = current;
+    else
+      dst = current;
+
+    prev = current;
+    src  = src->next;
+  }
+
+  return (dst);
+}
+
+
 /*
  * 'httpAddrFreeList()' - Free an address list.
  *
- * @since CUPS 1.2/Mac OS X 10.5@
+ * @since CUPS 1.2/OS X 10.5@
  */
 
 void
@@ -211,7 +385,7 @@ httpAddrFreeList(
 /*
  * 'httpAddrGetList()' - Get a list of addresses for a hostname.
  *
- * @since CUPS 1.2/Mac OS X 10.5@
+ * @since CUPS 1.2/OS X 10.5@
  */
 
 http_addrlist_t	*			/* O - List of addresses or NULL */
@@ -222,30 +396,21 @@ httpAddrGetList(const char *hostname,	/* I - Hostname, IP address, or NULL for p
   http_addrlist_t	*first,		/* First address in list */
 			*addr,		/* Current address in list */
 			*temp;		/* New address */
-  static int	need_res_init = 0;	/* Do we need to do a res_init? */
 
 
-#ifdef HAVE_RES_INIT
- /*
-  * STR #2920: Initialize resolver after failure in cups-polld
-  *
-  * If the previous lookup failed, re-initialize the resolver to prevent
-  * temporary network errors from persisting.  This *should* be handled by
-  * the resolver libraries, but apparently the glibc folks do not agree.
-  *
-  * We set a flag at the end of this function if we encounter an error that
-  * requires reinitialization of the resolver functions.  We then call
-  * res_init() if the flag is set on the next call here or in httpAddrLookup().
-  */
-
-  if (need_res_init)
-  {
-    res_init();
-
-    need_res_init = 0;
-  }
-#endif /* HAVE_RES_INIT */
-
+#ifdef DEBUG
+  _cups_debug_printf("httpAddrGetList(hostname=\"%s\", family=AF_%s, "
+                     "service=\"%s\")\n",
+		     hostname ? hostname : "(nil)",
+		     family == AF_UNSPEC ? "UNSPEC" :
+#  ifdef AF_LOCAL
+	                 family == AF_LOCAL ? "LOCAL" :
+#  endif /* AF_LOCAL */
+#  ifdef AF_INET6
+	                 family == AF_INET6 ? "INET6" :
+#  endif /* AF_INET6 */
+	                 family == AF_INET ? "INET" : "???", service);
+#endif /* DEBUG */
 
  /*
   * Lookup the address the best way we can...
@@ -262,6 +427,7 @@ httpAddrGetList(const char *hostname,	/* I - Hostname, IP address, or NULL for p
 
     if ((first = (http_addrlist_t *)calloc(1, sizeof(http_addrlist_t))) != NULL)
     {
+      addr = first;
       first->addr.un.sun_family = AF_LOCAL;
       strlcpy(first->addr.un.sun_path, hostname, sizeof(first->addr.un.sun_path));
     }
@@ -347,6 +513,7 @@ httpAddrGetList(const char *hostname,	/* I - Hostname, IP address, or NULL for p
 	  if (!temp)
 	  {
 	    httpAddrFreeList(first);
+	    _cupsSetError(IPP_STATUS_ERROR_INTERNAL, strerror(errno), 0);
 	    return (NULL);
 	  }
 
@@ -376,8 +543,10 @@ httpAddrGetList(const char *hostname,	/* I - Hostname, IP address, or NULL for p
 
       freeaddrinfo(results);
     }
-    else if (error == EAI_FAIL)
-      need_res_init = 1;
+    else
+    {
+      _cupsSetError(IPP_STATUS_ERROR_INTERNAL, gai_strerror(error), 0);
+    }
 
 #else
     if (hostname)
@@ -439,9 +608,10 @@ httpAddrGetList(const char *hostname,	/* I - Hostname, IP address, or NULL for p
 	    return (NULL);
 
           first->addr.ipv4.sin_family = AF_INET;
-          first->addr.ipv4.sin_addr.s_addr = htonl(((((((ip[0] << 8) |
-	                                               ip[1]) << 8) |
-						     ip[2]) << 8) | ip[3]));
+          first->addr.ipv4.sin_addr.s_addr = htonl((((((((unsigned)ip[0] << 8) |
+	                                               (unsigned)ip[1]) << 8) |
+						     (unsigned)ip[2]) << 8) |
+						   (unsigned)ip[3]));
           first->addr.ipv4.sin_port = htons(portnum);
 	}
       }
@@ -495,8 +665,13 @@ httpAddrGetList(const char *hostname,	/* I - Hostname, IP address, or NULL for p
 	  addr = temp;
 	}
       }
-      else if (h_errno == NO_RECOVERY)
-        need_res_init = 1;
+      else
+      {
+        if (h_errno == NO_RECOVERY)
+          cg->need_res_init = 1;
+
+	_cupsSetError(IPP_STATUS_ERROR_INTERNAL, hstrerror(h_errno), 0);
+      }
     }
 #endif /* HAVE_GETADDRINFO */
   }
@@ -534,6 +709,8 @@ httpAddrGetList(const char *hostname,	/* I - Hostname, IP address, or NULL for p
     else
     {
       httpAddrFreeList(first);
+
+      _cupsSetError(IPP_STATUS_ERROR_INTERNAL, _("Unknown service name."), 1);
       return (NULL);
     }
 
@@ -556,6 +733,7 @@ httpAddrGetList(const char *hostname,	/* I - Hostname, IP address, or NULL for p
 	temp = (http_addrlist_t *)calloc(1, sizeof(http_addrlist_t));
 	if (!temp)
 	{
+	  _cupsSetError(IPP_STATUS_ERROR_INTERNAL, strerror(errno), 0);
 	  httpAddrFreeList(first);
 	  return (NULL);
 	}
@@ -584,6 +762,7 @@ httpAddrGetList(const char *hostname,	/* I - Hostname, IP address, or NULL for p
 	temp = (http_addrlist_t *)calloc(1, sizeof(http_addrlist_t));
 	if (!temp)
 	{
+	  _cupsSetError(IPP_STATUS_ERROR_INTERNAL, strerror(errno), 0);
 	  httpAddrFreeList(first);
 	  return (NULL);
 	}
@@ -615,6 +794,7 @@ httpAddrGetList(const char *hostname,	/* I - Hostname, IP address, or NULL for p
 	temp = (http_addrlist_t *)calloc(1, sizeof(http_addrlist_t));
 	if (!temp)
 	{
+	  _cupsSetError(IPP_STATUS_ERROR_INTERNAL, strerror(errno), 0);
 	  httpAddrFreeList(first);
 	  return (NULL);
 	}
@@ -638,6 +818,7 @@ httpAddrGetList(const char *hostname,	/* I - Hostname, IP address, or NULL for p
 	temp = (http_addrlist_t *)calloc(1, sizeof(http_addrlist_t));
 	if (!temp)
 	{
+	  _cupsSetError(IPP_STATUS_ERROR_INTERNAL, strerror(errno), 0);
 	  httpAddrFreeList(first);
 	  return (NULL);
 	}
@@ -663,5 +844,5 @@ httpAddrGetList(const char *hostname,	/* I - Hostname, IP address, or NULL for p
 
 
 /*
- * End of "$Id: http-addrlist.c 1702 2011-12-30 07:41:10Z mike $".
+ * End of "$Id: http-addrlist.c 1726 2016-01-02 21:46:20Z msweet $".
  */
