@@ -1,25 +1,13 @@
 /*
- * "$Id: image.cxx,v 1.11.2.15 2001/10/20 21:49:22 mike Exp $"
+ * "$Id: image.cxx 1729 2016-01-02 22:58:41Z msweet $"
  *
  *   Image handling routines for HTMLDOC, a HTML document processing program.
  *
- *   Copyright 1997-2001 by Easy Software Products.
+ *   Copyright 2011 by Michael R Sweet.
+ *   Copyright 1997-2010 by Easy Software Products.  All rights reserved.
  *
- *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "COPYING.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: ESP Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: info@easysw.com
- *         WWW: http://www.easysw.com
+ *   This program is free software.  Distribution and use rights are outlined in
+ *   the file "COPYING.txt".
  *
  * Contents:
  *
@@ -38,7 +26,7 @@
  *   image_load_jpeg()    - Load a JPEG image file.
  *   image_load_png()     - Load a PNG image file.
  *   image_need_mask()    - Allocate memory for the image mask...
- *   image_set_mask()     - Clear a bit in the image mask.
+ *   image_set_mask()     - Set a bit in the image mask.
  *   jpeg_error_handler() - Handle JPEG errors by not exiting.
  *   read_word()          - Read a 16-bit unsigned integer.
  *   read_dword()         - Read a 32-bit unsigned integer.
@@ -107,8 +95,8 @@ static int	image_load_bmp(image_t *img, FILE *fp, int gray, int load_data);
 static int	image_load_gif(image_t *img, FILE *fp, int gray, int load_data);
 static int	image_load_jpeg(image_t *img, FILE *fp, int gray, int load_data);
 static int	image_load_png(image_t *img, FILE *fp, int gray, int load_data);
-static void	image_need_mask(image_t *img);
-static void	image_set_mask(image_t *img, int x, int y);
+static void	image_need_mask(image_t *img, int scaling = 1);
+static void	image_set_mask(image_t *img, int x, int y, uchar alpha = 0);
 
 static void		jpeg_error_handler(j_common_ptr);
 static int		read_long(FILE *fp);
@@ -234,14 +222,14 @@ gif_get_code(FILE *fp,		/* I - File to read from */
     * Just initialize the input buffer...
     */
 
-    curbit  = 0;
-    lastbit = 0;
-    done    = 0;
+    curbit    = 0;
+    lastbit   = 0;
+    last_byte = 0;
+    done      = 0;
 
     return (0);
   }
 
-
   if ((curbit + code_size) >= lastbit)
   {
    /*
@@ -294,7 +282,6 @@ gif_get_code(FILE *fp,		/* I - File to read from */
     lastbit   = last_byte * 8;
   }
 
-  ret = 0;
   for (ret = 0, i = curbit + code_size - 1, j = code_size;
        j > 0;
        i --, j --)
@@ -557,39 +544,46 @@ image_compare(image_t **img1,	/* I - First image */
  */
 
 void
-image_copy(const char *filename,/* I - Source file */
-           const char *destpath)/* I - Destination path */
+image_copy(const char *src,		/* I - Source file */
+           const char *realsrc,		/* I - Real source file */
+           const char *destpath)	/* I - Destination path */
 {
-  char	dest[255];		/* Destination file */
-  FILE	*in, *out;		/* Input/output files */
-  uchar	buffer[8192];		/* Data buffer */
-  int	nbytes;			/* Number of bytes in buffer */
+  char		dest[255];		/* Destination file */
+  FILE		*in, *out;		/* Input/output files */
+  uchar		buffer[8192];		/* Data buffer */
+  int		nbytes;			/* Number of bytes in buffer */
 
 
+  if (!src || !realsrc || !destpath)
+    return;
+
  /*
   * Figure out the destination filename...
   */
 
-  if (strcmp(destpath, ".") == 0)
-    strcpy(dest, file_basename(filename));
+  if (!strcmp(destpath, "."))
+    strlcpy(dest, file_basename(src), sizeof(dest));
   else
-    sprintf(dest, "%s/%s", destpath, file_basename(filename));
+    snprintf(dest, sizeof(dest), "%s/%s", destpath, file_basename(src));
 
-  if (strcmp(dest, filename) == 0)
+  if (!strcmp(dest, realsrc))
     return;
 
  /*
   * Open files and copy...
   */
 
-  if ((filename = file_find(Path, filename)) == NULL)
-    return;
-
-  if ((in = fopen(filename, "rb")) == NULL)
+  if ((in = fopen(realsrc, "rb")) == NULL)
+  {
+    progress_error(HD_ERROR_READ_ERROR, "Unable to open \"%s\" - %s",
+                   realsrc, strerror(errno));
     return;
+  }
 
   if ((out = fopen(dest, "wb")) == NULL)
   {
+    progress_error(HD_ERROR_READ_ERROR, "Unable to create \"%s\" - %s",
+                   dest, strerror(errno));
     fclose(in);
     return;
   }
@@ -597,6 +591,8 @@ image_copy(const char *filename,/* I - Source file */
   while ((nbytes = fread(buffer, 1, sizeof(buffer), in)) > 0)
     fwrite(buffer, 1, nbytes, out);
 
+  progress_error(HD_ERROR_NONE, "BYTES: %ld", ftell(out));
+
   fclose(in);
   fclose(out);
 }
@@ -631,7 +627,7 @@ image_find(const char *filename,/* I - Name of image file */
 
   if (num_images > 0)
   {
-    strcpy(key.filename, filename);
+    strlcpy(key.filename, filename, sizeof(key.filename));
     keyptr = &key;
 
     match = (image_t **)bsearch(&keyptr, images, num_images, sizeof(image_t *),
@@ -706,6 +702,9 @@ image_load(const char *filename,/* I - Name of image file */
            int        gray,	/* I - 0 = color, 1 = grayscale */
            int        load_data)/* I - 1 = load image data, 0 = just info */
 {
+#ifdef DEBUG
+  int		i;		/* Looping var */
+#endif // DEBUG
   FILE		*fp;		/* File pointer */
   uchar		header[16];	/* First 16 bytes of file */
   image_t	*img,		/* New image buffer */
@@ -727,13 +726,17 @@ image_load(const char *filename,/* I - Name of image file */
   if (filename[0] == '\0')	/* Microsoft VC++ runtime bug workaround... */
     return (NULL);
 
+  DEBUG_printf(("image_load(filename=\"%s\", gray=%d, load_data=%d)\n",
+                filename, gray, load_data));
+  DEBUG_printf(("Path = \"%s\"\n", Path));
+
  /*
   * See if we've already loaded it...
   */
 
   if (num_images > 0)
   {
-    strcpy(key.filename, filename);
+    strlcpy(key.filename, filename, sizeof(key.filename));
     keyptr = &key;
 
     match = (image_t **)bsearch(&keyptr, images, num_images, sizeof(image_t *),
@@ -774,6 +777,20 @@ image_load(const char *filename,/* I - Name of image file */
     return (NULL);
   }
 
+#ifdef DEBUG
+  printf("Header for \"%s\" (%s): \"", filename, realname);
+
+  for (i = 0; i < sizeof(header); i ++)
+    if (header[i] < ' ' || header[i] >= 127)
+      printf("\\x%02X", header[i]);
+    else
+      putchar(header[i]);
+
+  puts("\"\n");
+
+  printf("match = %p\n", match);
+#endif // DEBUG
+
   rewind(fp);
 
   // See if the images array needs to be resized...
@@ -814,7 +831,7 @@ image_load(const char *filename,/* I - Name of image file */
 
     images[num_images] = img;
 
-    strcpy(img->filename, filename);
+    strlcpy(img->filename, filename, sizeof(img->filename));
     img->use = 1;
   }
   else
@@ -828,12 +845,12 @@ image_load(const char *filename,/* I - Name of image file */
     status = image_load_bmp(img, fp, gray, load_data);
   else if (memcmp(header, "\211PNG", 4) == 0)
     status = image_load_png(img, fp, gray, load_data);
-  else if (memcmp(header, "\377\330\377", 3) == 0 &&	/* Start-of-Image */
-	   header[3] >= 0xe0 && header[3] <= 0xef)	/* APPn */
+  else if (memcmp(header, "\377\330\377", 3) == 0)
     status = image_load_jpeg(img, fp, gray, load_data);
   else
   {
-    progress_error(HD_ERROR_BAD_FORMAT, "Unknown image file format for \"%s\"!", filename);
+    progress_error(HD_ERROR_BAD_FORMAT, "Unknown image file format for \"%s\"!",
+                   file_rlookup(filename));
     fclose(fp);
     free(img);
     return (NULL);
@@ -843,7 +860,8 @@ image_load(const char *filename,/* I - Name of image file */
 
   if (status)
   {
-    progress_error(HD_ERROR_READ_ERROR, "Unable to load image file \"%s\"!", filename);
+    progress_error(HD_ERROR_READ_ERROR, "Unable to load image file \"%s\"!",
+                   file_rlookup(filename));
     if (!match)
       free(img);
     return (NULL);
@@ -920,6 +938,12 @@ image_load_bmp(image_t *img,	/* I - Image to load into */
   // Setup image and buffers...
   img->depth  = gray ? 1 : 3;
 
+  // If this image is indexed and we are writing an encrypted PDF file, bump the use count so
+  // we create an image object (Acrobat 6 bug workaround)
+  if (depth <= 8 && Encryption)
+    img->use ++;
+
+  // Return now if we only need the dimensions...
   if (!load_data)
     return (0);
 
@@ -1246,6 +1270,11 @@ image_load_gif(image_t *img,	/* I - Image pointer */
   img->height = (buf[9] << 8) | buf[8];
   ncolors     = 2 << (buf[10] & 0x07);
 
+  // If we are writing an encrypted PDF file, bump the use count so we create
+  // an image object (Acrobat 6 bug workaround)
+  if (Encryption)
+    img->use ++;
+
   if (buf[10] & GIF_COLORMAP)
     if (gif_read_cmap(fp, ncolors, cmap, &gray))
       return (-1);
@@ -1350,9 +1379,9 @@ image_load_jpeg(image_t *img,	/* I - Image pointer */
   cinfo.err = &jerr;
   jpeg_create_decompress(&cinfo);
   jpeg_stdio_src(&cinfo, fp);
-  jpeg_read_header(&cinfo, 1);
+  jpeg_read_header(&cinfo, (boolean)1);
 
-  cinfo.quantize_colors = 0;
+  cinfo.quantize_colors = FALSE;
 
   if (gray || cinfo.num_components == 1)
   {
@@ -1360,6 +1389,15 @@ image_load_jpeg(image_t *img,	/* I - Image pointer */
     cinfo.out_color_components = 1;
     cinfo.output_components    = 1;
   }
+  else if (cinfo.num_components != 3)
+  {
+    jpeg_destroy_decompress(&cinfo);
+
+    progress_error(HD_ERROR_BAD_FORMAT,
+                   "CMYK JPEG files are not supported! (%s)",
+		   file_rlookup(img->filename));
+    return (-1);
+  }
   else
   {
     cinfo.out_color_space      = JCS_RGB;
@@ -1414,22 +1452,54 @@ image_load_png(image_t *img,	/* I - Image pointer */
                int     gray,	/* I - 0 = color, 1 = grayscale */
                int     load_data)/* I - 1 = load image data, 0 = just info */
 {
-  int		i;	/* Looping var */
-  png_structp	pp;	/* PNG read pointer */
-  png_infop	info;	/* PNG info pointers */
-  png_bytep	*rows;	/* PNG row pointers */
-  uchar		*inptr,	/* Input pixels */
-		*outptr;/* Output pixels */
-  png_color_16	bg;	/* Background color */
-  float		rgb[3];	/* RGB color of background */
+  int		i, j;		/* Looping vars */
+  png_structp	pp;		/* PNG read pointer */
+  png_infop	info;		/* PNG info pointers */
+  int		depth;		/* Input image depth */
+  png_bytep	*rows;		/* PNG row pointers */
+  uchar		*inptr,		/* Input pixels */
+		*outptr;	/* Output pixels */
 
 
  /*
   * Setup the PNG data structures...
   */
 
-  pp   = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
+  pp = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
+  if (!pp)
+  {
+    progress_error(HD_ERROR_OUT_OF_MEMORY, "Unable to allocate memory for PNG file: %s",
+                   strerror(errno));
+    return (-1);
+  }
+
   info = png_create_info_struct(pp);
+  if (!info)
+  {
+    progress_error(HD_ERROR_OUT_OF_MEMORY, "Unable to allocate memory for PNG info: %s",
+                   strerror(errno));
+
+    png_destroy_read_struct(&pp, NULL, NULL);
+
+    return (-1);
+  }
+
+  rows = NULL;
+
+  if (setjmp(png_jmpbuf(pp)))
+  {
+    progress_error(HD_ERROR_BAD_FORMAT, "PNG file contains errors!");
+
+    png_destroy_read_struct(&pp, &info, NULL);
+
+    if (img != NULL && img->pixels != NULL)
+      free(img->pixels);
+
+    if (rows != NULL)
+      free(rows);
+
+    return (-1);
+  }
 
  /*
   * Initialize the PNG read "engine"...
@@ -1443,76 +1513,78 @@ image_load_png(image_t *img,	/* I - Image pointer */
 
   png_read_info(pp, info);
 
-  if (info->color_type == PNG_COLOR_TYPE_PALETTE)
+  if (png_get_color_type(pp, info) & PNG_COLOR_MASK_PALETTE)
+  {
     png_set_expand(pp);
 
-  if (info->color_type == PNG_COLOR_TYPE_GRAY)
-    img->depth = 1;
-  else
-    img->depth = gray ? 1 : 3;
-
-  img->width  = info->width;
-  img->height = info->height;
-
-  if (!load_data)
-  {
-    png_read_destroy(pp, info, NULL);
-    return (0);
+    // If we are writing an encrypted PDF file, bump the use count so we create
+    // an image object (Acrobat 6 bug workaround)
+    if (Encryption)
+      img->use ++;
   }
-
-  img->pixels = (uchar *)malloc(img->width * img->height * 3);
-
-  if (info->bit_depth < 8)
+  else if (png_get_bit_depth(pp, info) < 8)
   {
     png_set_packing(pp);
     png_set_expand(pp);
   }
-  else if (info->bit_depth == 16)
+  else if (png_get_bit_depth(pp, info) == 16)
     png_set_strip_16(pp);
 
- /*
-  * Handle transparency...
-  */
-
-  if (png_get_valid(pp, info, PNG_INFO_tRNS))
-    png_set_tRNS_to_alpha(pp);
-
-  if (BodyColor[0])
+  if (png_get_color_type(pp, info) & PNG_COLOR_MASK_COLOR)
   {
-   /*
-    * User-defined color...
-    */
-
-    get_color((uchar *)BodyColor, rgb);
-
-    bg.red   = (png_uint_16)(rgb[0] * 65535.0f + 0.5f);
-    bg.green = (png_uint_16)(rgb[1] * 65535.0f + 0.5f);
-    bg.blue  = (png_uint_16)(rgb[2] * 65535.0f + 0.5f);
+    depth      = 3;
+    img->depth = gray ? 1 : 3;
   }
   else
   {
-   /*
-    * Default to white...
-    */
-
-    bg.red   = 65535;
-    bg.green = 65535;
-    bg.blue  = 65535;
+    depth      = 1;
+    img->depth = 1;
   }
 
-  png_set_background(pp, &bg, PNG_BACKGROUND_GAMMA_SCREEN, 0, 1.0);
+  img->width  = png_get_image_width(pp, info);
+  img->height = png_get_image_height(pp, info);
+
+  if (png_get_color_type(pp, info) & PNG_COLOR_MASK_ALPHA)
+  {
+    if ((PSLevel == 0 && PDFVersion >= 14) || PSLevel == 3)
+      image_need_mask(img, 8);
+    else if (PSLevel == 0 && PDFVersion == 13)
+      image_need_mask(img, 2);
+    else
+      image_need_mask(img);
+
+    depth ++;
+  }
+
+#ifdef DEBUG
+  printf("color_type=0x%04x, depth=%d, img->width=%d, img->height=%d, img->depth=%d\n",
+         png_get_color_type(pp, info), depth, img->width, img->height, img->depth);
+  if (png_get_color_type(pp, info) & PNG_COLOR_MASK_COLOR)
+    puts("    COLOR");
+  else
+    puts("    GRAYSCALE");
+  if (png_get_color_type(pp, info) & PNG_COLOR_MASK_ALPHA)
+    puts("    ALPHA");
+  if (png_get_color_type(pp, info) & PNG_COLOR_MASK_PALETTE)
+    puts("    PALETTE");
+#endif // DEBUG
+
+  if (!load_data)
+  {
+    png_destroy_read_struct(&pp, &info, NULL);
+    return (0);
+  }
+
+  img->pixels = (uchar *)malloc(img->width * img->height * depth);
 
  /*
   * Allocate pointers...
   */
 
-  rows = (png_bytep *)calloc(info->height, sizeof(png_bytep));
+  rows = (png_bytep *)calloc(png_get_image_height(pp, info), sizeof(png_bytep));
 
-  for (i = 0; i < (int)info->height; i ++)
-    if (info->color_type == PNG_COLOR_TYPE_GRAY)
-      rows[i] = img->pixels + i * img->width;
-    else
-      rows[i] = img->pixels + i * img->width * 3;
+  for (i = 0; i < (int)png_get_image_height(pp, info); i ++)
+    rows[i] = img->pixels + i * img->width * depth;
 
  /*
   * Read the image, handling interlacing as needed...
@@ -1521,11 +1593,40 @@ image_load_png(image_t *img,	/* I - Image pointer */
   for (i = png_set_interlace_handling(pp); i > 0; i --)
     png_read_rows(pp, rows, NULL, img->height);
 
+ /*
+  * Generate the alpha mask as necessary...
+  */
+
+  if (png_get_color_type(pp, info) & PNG_COLOR_MASK_ALPHA)
+  {
+#ifdef DEBUG
+    for (inptr = img->pixels, i = 0; i < img->height; i ++)
+    {
+      for (j = 0; j < img->width; j ++, inptr += depth)
+        switch (depth)
+	{
+	  case 2 :
+	      printf(" %02X%02X", inptr[0], inptr[1]);
+	      break;
+	  case 4 :
+	      printf(" %02X%02X%02X%02X", inptr[0], inptr[1], inptr[2], inptr[3]);
+	      break;
+	}
+
+      putchar('\n');
+    }
+#endif // DEBUG
+
+    for (inptr = img->pixels + depth - 1, i = 0; i < img->height; i ++)
+      for (j = 0; j < img->width; j ++, inptr += depth)
+        image_set_mask(img, j, i, *inptr);
+  }
+
  /*
   * Reformat the data as necessary for the reader...
   */
 
-  if (gray && info->color_type != PNG_COLOR_TYPE_GRAY)
+  if (gray && png_get_color_type(pp, info) & PNG_COLOR_MASK_COLOR)
   {
    /*
     * Greyscale output needed...
@@ -1533,9 +1634,34 @@ image_load_png(image_t *img,	/* I - Image pointer */
 
     for (inptr = img->pixels, outptr = img->pixels, i = img->width * img->height;
          i > 0;
-         inptr += 3, outptr ++, i --)
+         inptr += depth, outptr ++, i --)
       *outptr = (31 * inptr[0] + 61 * inptr[1] + 8 * inptr[2]) / 100;
   }
+  else if (img->depth != depth)
+  {
+   /*
+    * Remove alpha from final array...
+    */
+
+    if (depth == 4)
+    {
+      for (inptr = img->pixels, outptr = img->pixels, i = img->width * img->height;
+           i > 0;
+           inptr ++, i --)
+      {
+        *outptr++ = *inptr++;
+        *outptr++ = *inptr++;
+        *outptr++ = *inptr++;
+      }
+    }
+    else
+    {
+      for (inptr = img->pixels, outptr = img->pixels, i = img->width * img->height;
+           i > 0;
+           inptr ++, i --)
+        *outptr++ = *inptr++;
+    }
+  }
 
  /*
   * Free memory and return...
@@ -1544,7 +1670,7 @@ image_load_png(image_t *img,	/* I - Image pointer */
   free(rows);
 
   png_read_end(pp, info);
-  png_read_destroy(pp, info, NULL);
+  png_destroy_read_struct(&pp, &info, NULL);
 
   return (0);
 }
@@ -1555,7 +1681,8 @@ image_load_png(image_t *img,	/* I - Image pointer */
  */
 
 static void
-image_need_mask(image_t *img)	/* I - Image to add mask to */
+image_need_mask(image_t *img,	/* I - Image to add mask to */
+                int     scaling)/* I - Scaling for mask image */
 {
   int	size;			/* Byte size of mask image */
 
@@ -1563,41 +1690,83 @@ image_need_mask(image_t *img)	/* I - Image to add mask to */
   if (img == NULL || img->mask != NULL)
     return;
 
- /* 
+ /*
   * Figure out the size of the mask image, and then allocate and set all the
   * bits needed...
   */
 
-  img->maskwidth = (img->width + 7) / 8;
-  size           = img->maskwidth * img->height;
-  
+  img->maskscale = scaling;
+
+  if (scaling == 8)
+  {
+    // Alpha image
+    img->maskwidth = img->width;
+    size           = img->width * img->height;
+  }
+  else
+  {
+    // Alpha mask
+    img->maskwidth = (img->width * scaling + 7) / 8;
+    size           = img->maskwidth * img->height * scaling + 1;
+  }
+
   img->mask = (uchar *)calloc(size, 1);
 }
 
 
 /*
- * 'image_set_mask()' - Clear a bit in the image mask.
+ * 'image_set_mask()' - Set a bit in the image mask.
  */
 
 static void
 image_set_mask(image_t *img,	/* I - Image to operate on */
                int     x,	/* I - X coordinate */
-               int     y)	/* I - Y coordinate */
+               int     y,	/* I - Y coordinate */
+	       uchar   alpha)	/* I - Alpha value */
 {
+  int		i, j;		/* Looping vars */
   uchar		*maskptr;	/* Pointer into mask image */
   static uchar	masks[8] =	/* Masks for each bit */
 		{
 		  0x80, 0x40, 0x20, 0x10,
 		  0x08, 0x04, 0x02, 0x01
 		};
+  static uchar	dither[4][4] = // Simple 4x4 clustered-dot dither
+		{
+		  { 0,  2,  15, 6 },
+		  { 4,  12, 9,  11 },
+		  { 14, 7,  1,  3 },
+		  { 8,  10, 5,  13 }
+	        };
 
 
   if (img == NULL || img->mask == NULL || x < 0 || x >= img->width ||
       y < 0 || y > img->height)
     return;
 
-  maskptr  = img->mask + y * img->maskwidth + x / 8;
-  *maskptr |= masks[x & 7];
+  if (img->maskscale == 8)
+  {
+    // Store the alpha value directly...
+    if (PSLevel)
+      img->mask[y * img->maskwidth + x] = 255 - alpha;
+    else
+      img->mask[y * img->maskwidth + x] = alpha;
+  }
+  else
+  {
+    // Store an alpha mask...
+    x *= img->maskscale;
+    y *= img->maskscale;
+    alpha >>= 4;
+
+    for (i = 0; i < img->maskscale; i ++, y ++, x -= img->maskscale)
+      for (j = 0; j < img->maskscale; j ++, x ++)
+      {
+	maskptr  = img->mask + y * img->maskwidth + x / 8;
+	if (alpha <= dither[x & 3][y & 3])
+	  *maskptr |= masks[x & 7];
+      }
+  }
 }
 
 
@@ -1691,5 +1860,5 @@ read_long(FILE *fp)               /* I - File to read from */
 
 
 /*
- * End of "$Id: image.cxx,v 1.11.2.15 2001/10/20 21:49:22 mike Exp $".
+ * End of "$Id: image.cxx 1729 2016-01-02 22:58:41Z msweet $".
  */
