@@ -1,25 +1,13 @@
 /*
- * "$Id: progress.cxx,v 1.6.2.8 2001/09/21 19:59:44 mike Exp $"
+ * "$Id: progress.cxx 1668 2011-12-23 06:23:11Z mike $"
  *
  *   Progress functions for HTMLDOC, a HTML document processing program.
  *
- *   Copyright 1997-2001 by Easy Software Products.
+ *   Copyright 2011 by Michael R Sweet.
+ *   Copyright 1997-2010 by Easy Software Products.  All rights reserved.
  *
- *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "COPYING.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: ESP Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: info@easysw.com
- *         WWW: http://www.easysw.com
+ *   This program is free software.  Distribution and use rights are outlined in
+ *   the file "COPYING.txt".
  *
  * Contents:
  *
@@ -40,6 +28,13 @@
 #  include <FL/fl_ask.H>
 #endif // HAVE_LIBFLTK
 
+#ifdef WIN32
+#  define getpid	GetCurrentProcessId
+static FILE	*error_log = NULL;
+#else
+#  include <unistd.h>
+#endif // WIN32
+
 
 /*
  * Local globals...
@@ -61,11 +56,14 @@ progress_error(HDerror    error,	/* I - Error number */
   char		text[2048];		/* Formatted text string */
 
 
+  if (error == HD_ERROR_HTML_ERROR && !StrictHTML)
+    return;
+
   if (error)
     Errors ++;
 
   va_start(ap, format);
-  vsprintf(text, format, ap);
+  vsnprintf(text, sizeof(text), format, ap);
   va_end(ap);
 
 #ifdef HAVE_LIBFLTK
@@ -78,15 +76,54 @@ progress_error(HDerror    error,	/* I - Error number */
   }
 #endif /* HAVE_LIBFLTK */
 
+#ifdef WIN32
+  // IIS doesn't separate stderr from stdout, so we have to send CGI errors
+  // to a separate event log...
+  if (CGIMode)
+  {
+    if (!error_log)
+    {
+      // Append messages to a file called "htmldoc.log"...
+      char	tmppath[1024];		// Buffer for temp dir
+      char	filename[1024];		// htmldoc.log filename
+
+
+      GetTempPath(sizeof(tmppath), tmppath);
+      snprintf(filename, sizeof(filename), "%s/htmldoc.log", tmppath);
+
+      error_log = fopen(filename, "a");
+    }
+
+    if (error_log)
+    {
+      fprintf(error_log, "HTMLDOC(%d) ", (int)getpid());
+
+      if (error)
+	fprintf(error_log, "ERR%03d: %s\n", error, text);
+      else
+	fprintf(error_log, "%s\n", text);
+
+      fflush(error_log);
+    }
+
+    return;
+  }
+#endif // WIN32
+
   if (Verbosity >= 0)
   {
     if (progress_visible)
       fprintf(stderr, "\r%-79.79s\r", "");
 
+    if (CGIMode)
+      fprintf(stderr, "HTMLDOC(%d) ", (int)getpid());
+
     if (error)
       fprintf(stderr, "ERR%03d: %s\n", error, text);
     else
       fprintf(stderr, "%s\n", text);
+
+    fflush(stderr);
   }
 }
 
@@ -106,6 +143,9 @@ progress_hide(void)
   }
 #endif /* HAVE_LIBFLTK */
 
+  if (CGIMode)
+    return;
+
   if (Verbosity > 0)
   {
     fprintf(stderr, "\r%-79.79s\r", "");
@@ -129,7 +169,7 @@ progress_show(const char *format,	/* I - Printf-style format string */
 
 
   va_start(ap, format);
-  vsprintf(text, format, ap);
+  vsnprintf(text, sizeof(text), format, ap);
   va_end(ap);
 
 #ifdef HAVE_LIBFLTK
@@ -140,9 +180,20 @@ progress_show(const char *format,	/* I - Printf-style format string */
   }
 #endif /* HAVE_LIBFLTK */
 
+  if (CGIMode)
+  {
+    if (Verbosity > 0)
+    {
+      fprintf(stderr, "HTMLDOC(%d) INFO: %s\n", (int)getpid(), text);
+      fflush(stderr);
+    }
+
+    return;
+  }
+
   if (Verbosity > 0)
   {
-    fprintf(stderr, "\r%-79s", text);
+    fprintf(stderr, "\r%-79.79s", text);
     fflush(stderr);
   }
 
@@ -168,5 +219,5 @@ progress_update(int percent)	/* I - Percent complete */
 
 
 /*
- * End of "$Id: progress.cxx,v 1.6.2.8 2001/09/21 19:59:44 mike Exp $".
+ * End of "$Id: progress.cxx 1668 2011-12-23 06:23:11Z mike $".
  */
