@@ -1,10 +1,10 @@
 /*
- * "$Id: toc.cxx 1668 2011-12-23 06:23:11Z mike $"
+ * "$Id: toc.cxx 1724 2014-03-16 02:49:18Z msweet $"
  *
  *   Table of contents generator for HTMLDOC, a HTML document processing
  *   program.
  *
- *   Copyright 2011 by Michael R Sweet.
+ *   Copyright 2011, 2014 by Michael R Sweet.
  *   Copyright 1997-2010 by Easy Software Products.  All rights reserved.
  *
  *   This program is free software.  Distribution and use rights are outlined in
@@ -28,6 +28,7 @@
  * Local functions...
  */
 
+static void	add_heading(tree_t *toc, tree_t *heading);
 static void	parse_tree(tree_t *t);
 
 
@@ -95,7 +96,7 @@ toc_build(tree_t *tree)		/* I - Document tree */
  * 'add_heading()' - Add heading records to the given toc entry...
  */
 
-void
+static void
 add_heading(tree_t *toc,	/* I - Table of contents */
             tree_t *heading)	/* I - Heading entry */
 {
@@ -387,5 +388,5 @@ parse_tree(tree_t *t)		/* I - Document tree */
 
 
 /*
- * End of "$Id: toc.cxx 1668 2011-12-23 06:23:11Z mike $".
+ * End of "$Id: toc.cxx 1724 2014-03-16 02:49:18Z msweet $".
  */
